#include <cassert>
#include <cstdio>
#include <cstring>
#include <vector>
#include <algorithm>
using namespace std;

const int N = 2000;
const int INF = 1000000000;

int n, weight[N], parent[N], answer[N + 1];
vector <int> adjacent[N];

int find(int i) {
    return parent[i] == -1 ? i : find(parent[i]);
}

bool mark[N];
int size[N], max_tree[N];
vector <int> nodes;
int minimum[N + 1][N + 1];

#define foreach(i, v) for (__typeof((v).begin()) i = (v).begin(); i != (v).end(); ++ i)

void dfs(int p, int u) {
    size[u] = 1;
    max_tree[u] = 0;
    nodes.push_back(u);
    foreach (iter, adjacent[u]) {
        int v = *iter;
        if (v != p && !mark[v]) {
            dfs(u, v);
            size[u] += size[v];
            max_tree[u] = max(max_tree[u], size[v]);
        }
    }
}

void divide(int root) {
    nodes.clear();
    dfs(-1, root);
    int total = size[root];
    int new_root = -1;
    int min_cost = INF;
    foreach (iter, nodes) {
        int u = *iter;
        int ret = max(total - size[u], max_tree[u]);
        if (ret < min_cost) {
            new_root = u;
            min_cost = ret;
        }
    }
    nodes.clear();
    dfs(-1, new_root);
    int m = nodes.size();
    minimum[m][0] = 0;
    for (int i = 1; i <= m; ++ i) {
        minimum[m][i] = INF;
    }
    for (int i = m - 1; i >= 0; -- i) {
        int id = nodes[i];
        for (int j = 0; j <= m; ++ j) {
            minimum[i][j] = minimum[i + size[id]][j];
            if (j) {
                minimum[i][j] = min(minimum[i][j], minimum[i + 1][j - 1] + weight[id]);
            }
        }
    }
    for (int j = 1; j <= m; ++ j) {
        answer[j] = min(answer[j], minimum[1][j - 1] + weight[new_root]);
    }
    mark[new_root] = true;
    foreach (iter, adjacent[new_root]) {
        int v = *iter;
        if (!mark[v]) {
            divide(v);
        }
    }
}

int main() {
    assert(scanf("%d", &n) == 1);
    assert(1 <= n && n <= N);
    for (int i = 0; i < n; ++ i) {
        assert(scanf("%d", weight + i) == 1);
        assert(1 <= weight[i] && weight[i] <= 1000000);
    }
    memset(parent, -1, sizeof(parent));
    for (int i = 0; i < n - 1; ++ i) {
        int a, b;
        assert(scanf("%d%d", &a, &b) == 2);
        a --;
        b --;
        adjacent[a].push_back(b);
        adjacent[b].push_back(a);
        assert(find(a) != find(b));
        parent[find(a)] = find(b);
    }
    for (int i = 1; i <= n; ++ i) {
        answer[i] = INF;
    }
    memset(mark, 0, sizeof(mark));
    divide(0);
    for (int i = 1; i <= n; ++ i) {
        printf("%d%c", answer[i], i == n ? '\n' : ' ' );
    }
    return 0;
}
