#include <cassert>
#include <cstdio>
#include <cstring>
#include <algorithm>
using namespace std;

const int N = 100000;

int n, q, parent[N];

int find(int i) {
    return parent[i] == -1 ? i : (parent[i] = find(parent[i]));
}

int edge_count, first_edge[N], to[N << 1], length[N << 1], next_edge[N << 1];

const int INF = 100000000;

int down[N][2], limit[2];

void add_edge(int u, int v, int w) {
    to[edge_count] = v;
    length[edge_count] = w;
    next_edge[edge_count] = first_edge[u];
    first_edge[u] = edge_count ++;
}

#define UPDATE(x, a) x = max(x, a)

void dfs(int p, int u) {
    down[u][0] = 0;
    down[u][1] = -INF;
    for (int iter = first_edge[u]; iter != -1; iter = next_edge[iter]) {
        int v = to[iter];
        int l = length[iter];
        if (v != p) {
            dfs(u, v);
            for (int x = 0; x < 2; ++ x) {
                for (int y = 0; y < 2; ++ y) {
                    UPDATE(limit[x + y + l & 1], down[u][x] + down[v][y] + l);
                }
            }
            for (int y = 0; y < 2; ++ y) {
                UPDATE(down[u][y + l & 1], down[v][y] + l);
            }
        }
    }
}

int main() {
    assert(scanf("%d%d", &n, &q) == 2);
    assert(1 <= n && n <= N);
    assert(1 <= q && q <= N);
    memset(parent, -1, sizeof(parent));
    edge_count = 0;
    memset(first_edge, -1, sizeof(first_edge));
    for (int i = 0; i < n - 1; ++ i) {
        int a, b, c;
        assert(scanf("%d%d%d", &a, &b, &c) == 3);
        assert(1 <= a && a <= n);
        assert(1 <= b && b <= n);
        assert(1 <= c && c <= 2);
        a --;
        b --;
        assert(find(a) != find(b));
        parent[find(a)] = find(b);
        add_edge(a, b, c);
        add_edge(b, a, c);
    }
    limit[0] = limit[1] = -INF;
    dfs(-1, 0);
    while (q --) {
        int l;
        assert(scanf("%d", &l) == 1);
        if (l < 0) {
            puts("No");
        } else {
            int t = l % 2;
            puts(l <= limit[t] ? "Yes" : "No");
        }
    }
    return 0;
}
